<?php 
/**
 * @package     Uso da biblioteca de sessao do Framework Joomla
 * @subpackage  Classe de exemplo de sessão 
 * @author		Johnny Salazar Reidel
 * @email		johnny@noboss.com.br
 * @copyright   No Boss Tecnologia
 *
 * @note		
 * - Essa classe utiliza todos os recursos apresentados nos demais arquivos de exemplo de sessão,
 * mas de maneira estruturada
 */

// Bloqueia acesso direto ao arquivo
if(!defined("ACESSO_NOBOSS")){ exit("Acesso não permitido"); }

// Classe principal da sessao
use Joomla\Session\Session;
// Classe banco de dados da sessao
use Joomla\Session\Storage\Database;
// Classe input
use Joomla\Input\Input;

/**
 * Classe SessionUser que estende a classe de sessão do Joomla
 */
class SessionUser extends Session
{
	protected $host;
	protected $db;
	protected $name;
	protected $id;
	protected $cookie_domain = '';
	protected $cookie_path = '/';
	protected $expire = 1200; // Tempo da sessão em segundos
	protected $username;
	protected $session_criptografia = '';
	
	/*
	 * Metodo construtor
	 * 
	 * @param 	Array		$options 		Dados necessários para criação da sessão
	 * @param 	Boolean		$new 			Nova sessão ou acesso a sessão existente
	 *
	 * @return	Boolean 	True em caso de sucesso e false em caso de erro
	 */
	public function __construct($options, $new = true)
	{
		// Armazena informações na classe
		foreach ($options as $key => $value) {
			$this->$key = $value;
		}

		// Acesso em sessão já existente
		if (!$new){
			// Cookies da sessao não existem: retorna false
			if (!($dadosCookies = $this->checksCookies($this->session_criptografia))){
				// Se Id da sessão estiver definido, destroy sessão
				if ($this->id != ''){
					$this->destroy($this->id);
				}
				return false;
			}

			// Registra informações dos cookies no objeto da classe
			$this->id = $dadosCookies['idsessao'];
			$this->username = $dadosCookies['username'];
		} 

		// Define o nome da sessao
		$this->name = $this->session_criptografia.$this->username;

		// Instancia objeto input
		$input = new Input();

		// Prepara options
		$options = array('db' => $this->db, 
						 'name' => $this->name, 
						 'cookie_domain' => $this->cookie_domain,
						 'cookie_path' => $this->cookie_path, 
						 'expire' => $this->expire
						);

		// Se id da sessão estiver definido, passa no option
		if ($this->id != ''){
			$options['id'] = $this->id;
		}

		// Chama construtor parent da sessão do Joomla
		parent::__construct('Database', $options);

		// Inicializa a sessao
		$this->initialise($input);
		$this->start();

		// Acesso em sessão nova
		if ($new){
			// Erro ao salvar sessão no banco de dados
			if (!$this->saveSessionDb()){
				$this->destroy($this->id);
				return false;
			}

			//Cria cookie 'NoBossDefault' para armazenar usuario que servirá para iniciar a sessao
			@setrawcookie("NoBossDefault", $this->username, 0, $this->cookie_path, $this->cookie_domain);
		}

		return true;
	}

	/*
	 * Metodo que verifica se cookies de autenticação existem
	 * 
	 * @param 	String		$sessionCriptografia 	Criptografia utilizada no nome da sessão
	 *
	 * @return	mixed 		Array com dados dos cookies ou false caso não existam
	 */
	public static function checksCookies($sessionCriptografia){
		// Cookie do usuário não existe
		if (!($username = $_COOKIE["NoBossDefault"])){
			return false;
		}

		// Cookie da sessão não existe
		if (!($idSessao = $_COOKIE[md5($sessionCriptografia.$username)])){
			return false;
		}

		return array ('username' => $username, 'idsessao' => $idSessao);
	}

	/*
	 * Metodo que salva a sessão no banco de dados
	 *
	 * @return	Boolean 	True em caso de sucesso e false em caso de erro
	 */
	public function saveSessionDb(){
		// Prepara os dados para salvar sessão no banco de dados
		$dadosdbSession = new stdClass();
		$dadosdbSession->session_id = $this->getId();
		$dadosdbSession->time = $this->getExpire();
		$dadosdbSession->username = $this->username;

		// Cookies da sessao já existem: realiza update da sessao no banco
		if ($this->checksCookies($this->session_criptografia)){
			$result = $this->db->updateObject('#__session', $dadosdbSession, 'username', false); 
		}
		// Cria a sessão no banco
		else{
			$result = $this->db->insertObject('#__session', $dadosdbSession); 
		}

		$this->db->freeResult();

		if (!$result){
			return false;
		}

		return true;
	}

	/*
	 * Metodo que destrói a sessão, incluindo cookies e banco de dados
	 * 
	 * @param 	Integer		$idSessao 		Id da sessão que será destruida
	 *
	 * @return	boolean 		True caso tenha sido destruída e false em caso de erro
	 */
	public function destroy($idSessao){
		// Acessa parent para destruir sessao, inclusive no banco de dados
		if (parent::destroy($idSessao)){
			// Destrói o cookie que armazena o usuário
			@setrawcookie("NoBossDefault", "-1", time() - 86400, "/", $cookieDomain);
			return true;
		}
		return false;		
	}

	/*
	 * Metodo que verifica se a sessão está ativa: caso não esteja ela é destruida
	 *
	 * @return	boolean 	True caso ainda esteja ativa e false caso esteja inativa
	 */
	public function checkSessionActive(){
		// A sessão expirou, esta invativa ou com erro e deve ser encerrada
		if (!($this->getState() == 'active')){
			// Destroi a sessão, incluindo banco de dados e cookie
			$this->destroy($this->getId());
			return false;
		}
		return true;
	}

	/*
	 * Metodo que obtem o username do usuário na sessão
	 *
	 * @return	String 	Username do usuário ou string vazia caso não definido
	 */
	public function getUsername(){
		return $this->username;
	}

	/*
	 * Metodo que obtem em segundos o tempo que falta para a sessão expirar
	 *
	 * @note 	Para acessar essa função é necessário ter a sessão instanciada e isso faz com que o tempo
	 * 			da sessão seja renovado.
	 *
	 * @return	int 	Tempo em segundos que resta para expirar
	 */
	public function getTimeRemaining(){
		$curTime = $this->get('session.timer.now', 0);
		$maxTime = $this->get('session.timer.last', 0) + $this->getExpire();

		return $maxTime - $curTime;
	}

	/*
	 * Metodo que obtem em segundos o tempo que falta para a sessão expirar direto do banco de dados
	 *
	 * @note 	Essa função pode ser acessada sem ter a sessão instanciada e por isso o tempo que resta
	 *			para a sessão expirar é retornado sem renovar o mesmo.
	 *
	 * @return	int 	Tempo em segundos que resta para expirar
	 */
	public static function getTimeRemainingDb($db, $idSessao, $expire){
		
		// Instancia objeto banco de dados da sessão
		$sessionDb = new Database(array('db' => $db));
		$dadosSessaoDb = $sessionDb->read($idSessao);

		// Remove caracter que impede realizar o unserialize
		$dadosSessaoDb = str_replace('__default|', '', $dadosSessaoDb);
		$dadosSessaoDb = unserialize($dadosSessaoDb);

		// Time limite para a sessão expirar (time do último acesso + tempo definido para expirar)
		$maxTime = $dadosSessaoDb['session.timer.last'] + $expire;
		// Time corrente
		$curTime = time();

		$tempoRestante = $maxTime - $curTime;

		if ($tempoRestante < 0){
			$tempoRestante = 0;
		}

		return $tempoRestante;
	}

	/*
	 * Metodo garbage collector para remover do banco sessoes expiradas
	 *
	 * @param   integer  $lifetime  The maximum age of a session.
	 *
	 * @return  boolean  True on success, false otherwise.
	 */
	public function gc($lifetime = 1440){
		// Instancia objeto banco de dados da sessao
		$sessionDb = new Database(array('db' => $this->db));
		// Executa função garbage collector da classe instanciada
		return $sessionDb->gc($lifetime);
	}
	
}
?>
